

var MapManager = cc.Layer.extend({

    _active: true,
    _mapsize: cc.size(0,0),
    _tilesize : cc.size(0,0),
    _tag: 0,
    _mode : 0,
    _isScroll: true,

    _mapFileName : null,
    _tileMap : null,
    _mapLayer : [],
    _eventLayer : null, // イベントレイヤー
    _bgLayer : null,    // 地面レイヤー
    _bg2Layer : null,   // 雲レイヤー
    _bg3Layer : null,   // 遠景レイヤー

    _cameraType : 0,        // カメラタイプ 0:player追いかける。 1:強制スクロール
    _cameraSpeed : 240,
    _cameraLimitX : 0,
    _cameraLimitY : 0,

    _startPos: cc.p(0,0),

    _restartFlag : false,

//===================================================================
//  コンストラクタ
//===================================================================
ctor : function()
{
    this._super();

    this._active = false;
    this._isScroll = false;

    this._tileMap = null;
    this._mapLayer.length = 0;
    this._restartFlag = false;

    this.x = 0;
    this.y = 0;
    return this;
},

init : function(jsonData)
{
    this._mapFileName = jsonData.map_name;
    this._tileMap = null;
    this._isScroll = true;
    this._cameraSpeed = SCROLL_SPEED;

    var tex = cc.textureCache.addImage(jsonData.tex_name);
    //tex.setAliasTexParameters();

    var map = new cc.TMXTiledMap(jsonData.map_name[gAreaNum]);
    if (map != null)
    {
        this._tileMap = map;
        this._tileMap.x = 0;
        this._tileMap.y = 0;
        this.addChild(map);

        this._mapsize  = map.getMapSize();
        this._tilesize = map.getTileSize();
        this._mapWidth = this._mapsize.width * this._tilesize.width;
        this._mapHeight = this._mapsize.height * this._tilesize.height;
        this._cameraLimitX = this._mapWidth - GAME_WIDTH;
        this._cameraLimitY = this._mapHeight - GAME_HEIGHT;

        this._eventLayer = map.getLayer("イベント");
        this._bgLayer = map.getLayer("地形");
        this._bg2Layer = map.getLayer("雲");
        this._bg3Layer = map.getLayer("遠景");
        map.reorderChild(this._bgLayer,200);
        map.reorderChild(this._bg2Layer,100);
        map.reorderChild(this._bg3Layer,0);

        this._mapLayer.length = 0;
        this._mapLayer.push(this._bgLayer);
        this._mapLayer.push(this._bg2Layer);
        this._mapLayer.push(this._bg3Layer);

        this._cameraType = 0;

        gWarpPoint.length = 0;
        gEnemies.length = 0;
        gBaneObj.length = 0;
        gCheckPointList.length = 0;
        var str;
        for (var w=0; w<this._mapsize.width; w++)
        {
            for(var h=0; h<this._mapsize.height; h++)
            {
                k = this._eventLayer.getTileGIDAt(cc.p(w,h));
                if (k < 0) k = -1;

                if (k == TILE_CODE.WARP)
                {
                    // ワープポイント
                    this.makeWarpPoint(w,h);

                    // ワープポイントもチェックポイントと同じ
                    gCheckPointList.push(cc.p(w,h));
                }
                else if (k == TILE_CODE.GOAL)
                {
                    // ゴールポイント
                    this.makeGoalPoint(w,h);
                }
                else if ( k == TILE_CODE.BANE)
                {
                    var pos = this.convertToMapPos(cc.p(w,h));
                    var node = Spr_Bane.getOrCreate(6, pos, this._tileMap);
                }
                else if ( k == TILE_CODE.SAVE)
                {
                    var pos = this.convertToMapPos(cc.p(w,h));
                    pos.y += this._tilesize.height / 2;
                    var node = Spr_Heart.getOrCreate(7, pos, this._tileMap, cc.p(w,h), this._bgLayer);
                }
                else if ( k >= TILE_CODE.ENEMY && k < TILE_CODE.ENEMY + 4)
                {
                    var pos = this.convertToMapPos(cc.p(w,h));
                    pos.y += this._tilesize.height / 2;
                    var node = Spr_Enemy.getOrCreate(k-TILE_CODE.ENEMY +2, pos, this._tileMap);
                    //this._tileMap.addChild(node);
                }
                else if ( k == TILE_CODE.CHECK)
                {
                    gCheckPointList.push(cc.p(w,h));
                }

            }
        }
        this.makeDeadPoint();

        this._eventLayer.visible = false;


this._cameraType = 1;

        //this.schedule(this.update);
    }
},
//===================================================================
// 破棄
//===================================================================
destroy : function()
{
    this.unschedule(this.update);

    this._active = false;

    //this.removeChild(this._tileMap, true);

    this._mapLayer.length = 0;
    this._tileMap = null;
    this._costLayer = null;
    this._eventLayer = null;
    this._mapFileName = null;
},
//===================================================================
// メイン
//===================================================================
update:function (dt)
{
    var _dt = gFrameRate;

    // ゲームオーバーだったらスクロールしない。
    if (!this._active || gLoseFlag || gWinFlag) return;

    switch(this._cameraType)
    {
    case 0:
        var map = this._tileMap;
        var pos = this.convertMapToScreenPos(cc.p(player.x, player.y));
        this.scroll(_dt);
        break;
    case 1:
        this.scroll(_dt);
        break;
    default:
        break;
    }
},
end : function()
{
    this._active = false;
    this.unschedule(this.update);
},
start : function()
{
    this._active = true;
    this._isScroll = true;
    this.schedule(this.update);
},
restart : function(srlpos)
{
    this._restartFlag = true;

    srlpos = -srlpos / this._tilesize.width;
    //srlpos -= 3;
    if (srlpos < 0) srlpos = 0;
    srlpos *= -this._tilesize.width;

    this._tileMap.x = srlpos;
    this._mapLayer[1].x = -srlpos / 4;
    this._mapLayer[2].x = -srlpos / 3;

    this._active = true;
    this._isScroll = true;
    this.schedule(this.update);
},
pause : function()
{
    cc.director.getScheduler().pauseTarget(this);
},
resume : function()
{
    cc.director.getScheduler().resumeTarget(this);
},
//===================================================================
//  マップスクロール
//===================================================================
scroll : function(dt)
{
    if (!this._isScroll) return;

    var map = this._tileMap;
    var speed = dt * this._cameraSpeed;    //@小数点四捨五入--> Math.round(dt * this._cameraSpeed);
    this._tileMap.x -= speed;
    if (-this._tileMap.x >= this._cameraLimitX)
    {
        this._isScroll = false;
        this._tileMap.x = -this._cameraLimitX;
        g_sharedGameScene.setupAreaClearDemo();
    }
    this._mapLayer[1].x = - this._tileMap.x / 4;
    this._mapLayer[2].x = - this._tileMap.x / 3;
},
//===================================================================
//  レイヤー取得
//===================================================================
getLayer : function(layername)
{
    return this._tileMap.getLayer(layername);
},

getBgTileNum : function(x,y)
{
    return this._bgLayer.getTileGIDAt(this.convertToTileMapPos(cc.p(x,y)));
},

//===================================================================
// x,y座標 ---> タイルマップの座標に変換
//===================================================================
convertToTileMapPos : function(pos)
{
    var tx = Math.floor(pos.x / this._tilesize.width);
    var ty = Math.floor(pos.y / this._tilesize.height);

    // タイルマップは左上が原点(0,0)なので、変換。
    ty = this._mapsize.height - ty - 1;

    return cc.p(tx,ty);
},
//===================================================================
// タイルマップの座標 ---> x,y座標
//===================================================================
convertToMapPos : function(tilepos)
{
    // タイルマップは左上が原点(0,0)なので、変換。
    var x = tilepos.x;
    var y = this._mapsize.height - tilepos.y - 1;
    x = x * this._tilesize.width + this._tilesize.width/2;
    y = y * this._tilesize.height;
    return cc.p(x,y);
},
//===================================================================
//  画面座標からワールド座標に変換
//===================================================================
convertScreenToMapPos : function(pos)
{
    var p = pos;
    var map = this._tileMap;
    p.x -= map.x;
    p.y -= map.y;

    // タイルマップの座標に変換〜ワールド座標に変換
    var tx = Math.floor(p.x / this._tilesize.width);
    p.x = tx * this._tilesize.width + this._tilesize.width/2;
    var ty = Math.floor(p.y / this._tilesize.height);
    p.y = ty * this._tilesize.height;
    return p;
},
// player専用
convertScreenToWorldPos : function(pos)
{
    var p = pos;
    var map = this._tileMap;
    p.x -= map.x;
    p.y -= map.y;
    return p;
},

//===================================================================
//  ワールド座標から画面座標に変換
//===================================================================
convertMapToScreenPos : function(pos)
{
    var p = pos;
    var map = this._tileMap;
    p.x += map.x;
    p.y += map.y;
    return p;
},

makeWarpPoint : function(w,h)
{
    var pos = this.convertToMapPos(cc.p(w,h));
    gWarpPoint.push(pos);

    
                                 
                                 
    var node = cc.Sprite.create(gWarpImage[gStageType]);
    node.attr({
        x: pos.x,
        y: pos.y,
        anchorX: 0.5,
        anchorY: 0,
    });
    this._tileMap.addChild(node);
},
makeGoalPoint : function(w,h)
{
    var pos = this.convertToMapPos(cc.p(w,h));
    gGoalPoint = pos;

    var node = cc.Sprite.create(gGoalImage[gStageType]);
    node.attr({
        x: pos.x+8,
        y: pos.y,
        anchorX: 0,
        anchorY: 0,
        scaleX: 0.5,
        scaleY: 0.5
    });
    this._tileMap.addChild(node);
},
makeDeadPoint: function()
{
    var pos, area;
    if (!gSpecialStage) {
        pos = gDeadPoint[gStageNum][0];
        area = gDeadPoint[gStageNum][1];
    }
    else {
        pos = gDeadPoint_SP[gStageNum][0];
        area = gDeadPoint_SP[gStageNum][1];
    }

    if (pos.x == 0 && pos.y == 0) return;

    if (area == gAreaNum)
    {
        var node = cc.Sprite.create("over_point.png");
        node.x = pos.x;
        node.y = pos.y;
        this._tileMap.addChild(node, Z_ORDER.PLAYER + 100, LAYER_TAG.DEADPOINT);
    }
}

});   // class end...
